local currentSettings = {
    lodDistanceBias = 0.2,
    pedLodBias = 0.2,
    vehicleLodBias = 0.2,
    particleLightMultiplier = 0.2,
    skyQuality = 0.5,
    pedDensity = 0.5,
    vehicleDensity = 0.5,
    reflectionQuality = 0.1,
    shadowQuality = 0.1,
    physicsQuality = 0.1,
    bloomEnabled = false,
    depthOfFieldEnabled = false,
    volumetricLightingEnabled = false,
    anisotropicFiltering = 2
}

local function logSettings(settings)
    for key, value in pairs(settings) do
        print(string.format("%s: %s", key, tostring(value)))
    end
end

local function optimizeSettings(playerId, fps)
    if fps < 30 then
        currentSettings.lodDistanceBias = 0.1
        currentSettings.pedLodBias = 0.1
        currentSettings.vehicleLodBias = 0.1
        currentSettings.particleLightMultiplier = 0.1
        currentSettings.skyQuality = 0.3
        currentSettings.pedDensity = 0.3
        currentSettings.vehicleDensity = 0.3
    elseif fps >= 30 and fps < 60 then
        currentSettings.lodDistanceBias = 0.3
        currentSettings.pedLodBias = 0.3
        currentSettings.vehicleLodBias = 0.3
        currentSettings.particleLightMultiplier = 0.3
        currentSettings.skyQuality = 0.5
        currentSettings.pedDensity = 0.5
        currentSettings.vehicleDensity = 0.5
    else
        currentSettings.lodDistanceBias = 0.5
        currentSettings.pedLodBias = 0.5
        currentSettings.vehicleLodBias = 0.5
        currentSettings.particleLightMultiplier = 0.5
        currentSettings.skyQuality = 0.7
        currentSettings.pedDensity = 0.7
        currentSettings.vehicleDensity = 0.7
    end

    TriggerClientEvent('performance:applyGraphicsSettings', playerId, currentSettings)
    print(string.format("Optimized settings for player %s based on FPS: %d", playerId, fps))
end

RegisterNetEvent('performance:reportFPS')
AddEventHandler('performance:reportFPS', function(fps)
    local playerId = source
    print(string.format("Received FPS (%d) from player %s", fps, playerId))
    optimizeSettings(playerId, fps)
end)

RegisterNetEvent('performance:updateGraphicsSettings')
AddEventHandler('performance:updateGraphicsSettings', function(newSettings)
    if newSettings and type(newSettings) == 'table' then
        for key, value in pairs(newSettings) do
            if currentSettings[key] ~= nil then
                currentSettings[key] = value
            end
        end
        print("Updated graphics settings:")
        logSettings(currentSettings)
        TriggerClientEvent('performance:applyGraphicsSettings', source, currentSettings)
    else
        print("Invalid settings received.")
    end
end)

RegisterNetEvent('performance:applyGraphicsSettingsToAll')
AddEventHandler('performance:applyGraphicsSettingsToAll', function()
    local players = GetPlayers()

    if #players > 0 then
        for _, playerId in ipairs(players) do
            TriggerClientEvent('performance:applyGraphicsSettings', playerId, currentSettings)
        end
    else
        print("No players connected to apply settings.")
    end
end)

Citizen.CreateThread(function()
    while true do
        Citizen.Wait(1000) 
        local players = GetPlayers()

        if #players > 0 then
            TriggerEvent('performance:applyGraphicsSettingsToAll')
        else
            print("No players available to apply settings.")
        end
    end
end)
