local maxConnectionsPerIp = 5
local connectionAttempts = {}
local cooldownTime = 60 
local aiThreshold = 0.8 
local function analyzeConnectionPattern(ip)
    local attempts = connectionAttempts[ip]
    if not attempts then return false end

    local currentTime = os.time()
    local recentAttempts = 0
    for _, timestamp in ipairs(attempts.timestamps or {}) do
        if currentTime - timestamp < cooldownTime then
            recentAttempts = recentAttempts + 1
        end
    end

    local suspicionScore = recentAttempts / maxConnectionsPerIp
    return suspicionScore > aiThreshold
end

AddEventHandler('playerConnecting', function(playerName, setKickReason, deferrals)
    local ip = GetPlayerEndpoint(source)
    local currentTime = os.time()

    deferrals.defer()

    if not connectionAttempts[ip] then
        connectionAttempts[ip] = { count = 1, lastAttempt = currentTime, timestamps = {currentTime} }
    else
        local lastAttempt = connectionAttempts[ip].lastAttempt
        if currentTime - lastAttempt < cooldownTime then
            connectionAttempts[ip].count = connectionAttempts[ip].count + 1
            table.insert(connectionAttempts[ip].timestamps, currentTime)
        else
            connectionAttempts[ip].count = 1
            connectionAttempts[ip].timestamps = {currentTime}
        end
        connectionAttempts[ip].lastAttempt = currentTime
    end

    if analyzeConnectionPattern(ip) then
        setKickReason("Suspicious connection behavior detected. Please contact support.")
        deferrals.done()
        return
    end

    if connectionAttempts[ip].count > maxConnectionsPerIp then
        setKickReason("Too many connection attempts. Please try again later.")
        deferrals.done()
    else
        deferrals.done()
    end
end)
