local serverSettings = {
    lastUpdateTime = 0,
    updateInterval = 10000,
    serverLoad = 0, 
    optimalGraphicsSettings = {
        lodDistanceBias = 10,
        pedLodBias = 10,
        vehicleLodBias = 10,
        particleLightMultiplier = 1.0,
        skyQuality = 2,
        pedDensity = 5,
        vehicleDensity = 5
    }
}

local function estimateServerLoad()
    local playerCount = GetNumPlayerIndices()
    local currentTime = GetGameTimer()

    if playerCount > 50 then
        serverSettings.serverLoad = 3 
    elseif playerCount > 20 then
        serverSettings.serverLoad = 2  
    else
        serverSettings.serverLoad = 1 
    end

    if serverSettings.serverLoad == 3 then
        serverSettings.optimalGraphicsSettings.lodDistanceBias = 5
        serverSettings.optimalGraphicsSettings.pedLodBias = 5
        serverSettings.optimalGraphicsSettings.vehicleLodBias = 5
        serverSettings.optimalGraphicsSettings.particleLightMultiplier = 0.8
        serverSettings.optimalGraphicsSettings.skyQuality = 1
        serverSettings.optimalGraphicsSettings.pedDensity = 3
        serverSettings.optimalGraphicsSettings.vehicleDensity = 3
    elseif serverSettings.serverLoad == 2 then
        serverSettings.optimalGraphicsSettings.lodDistanceBias = 8
        serverSettings.optimalGraphicsSettings.pedLodBias = 8
        serverSettings.optimalGraphicsSettings.vehicleLodBias = 8
        serverSettings.optimalGraphicsSettings.particleLightMultiplier = 0.9
        serverSettings.optimalGraphicsSettings.skyQuality = 2
        serverSettings.optimalGraphicsSettings.pedDensity = 4
        serverSettings.optimalGraphicsSettings.vehicleDensity = 4
    else
        serverSettings.optimalGraphicsSettings.lodDistanceBias = 12
        serverSettings.optimalGraphicsSettings.pedLodBias = 12
        serverSettings.optimalGraphicsSettings.vehicleLodBias = 12
        serverSettings.optimalGraphicsSettings.particleLightMultiplier = 1.0
        serverSettings.optimalGraphicsSettings.skyQuality = 3
        serverSettings.optimalGraphicsSettings.pedDensity = 6
        serverSettings.optimalGraphicsSettings.vehicleDensity = 6
    end
end

local function autoOptimizeGraphics()
    estimateServerLoad()

    TriggerClientEvent('performance:applyGraphicsSettings', -1, 
        serverSettings.optimalGraphicsSettings.lodDistanceBias,
        serverSettings.optimalGraphicsSettings.pedLodBias,
        serverSettings.optimalGraphicsSettings.vehicleLodBias,
        serverSettings.optimalGraphicsSettings.particleLightMultiplier,
        serverSettings.optimalGraphicsSettings.skyQuality,
        serverSettings.optimalGraphicsSettings.pedDensity,
        serverSettings.optimalGraphicsSettings.vehicleDensity
    )
end

local function canUpdateGraphics()
    local currentTime = GetGameTimer()
    if currentTime - serverSettings.lastUpdateTime < serverSettings.updateInterval then
        return false
    end
    serverSettings.lastUpdateTime = currentTime
    return true
end

RegisterNetEvent('performance:updateGraphicsSettings')
AddEventHandler('performance:updateGraphicsSettings', function(lodDistanceBias, pedLodBias, vehicleLodBias, particleLightMultiplier, skyQuality, pedDensity, vehicleDensity)
    if not (lodDistanceBias and pedLodBias and vehicleLodBias and particleLightMultiplier and skyQuality and pedDensity and vehicleDensity) then
        print("Error: Invalid graphics settings received")
        return
    end
    if not canUpdateGraphics() then
        return
    end
    TriggerClientEvent('performance:applyGraphicsSettings', source, lodDistanceBias, pedLodBias, vehicleLodBias, particleLightMultiplier, skyQuality, pedDensity, vehicleDensity)
end)
Citizen.CreateThread(function()
    while true do
        Citizen.Wait(30000) 
        autoOptimizeGraphics() 
    end
end)
